import { mkdirSync, writeFileSync, statSync, readdirSync } from 'fs';
import { join, dirname } from 'path';
import { defineNuxtModule, createResolver } from '@nuxt/kit';

class SiteMapGenerator {
  constructor(input, opts) {
    this.urls = [];
    this.basePath = "";
    this.basePath = opts.basePath || "/";
    this.excludeAnyRegExp = opts.excludeAnyRegExp;
    for (const r of input) {
      const genURLs = this.generateUrls(
        r.route,
        r.data
      );
      this.urls.push(...genURLs);
    }
  }
  get urlCount() {
    return this.urls.length;
  }
  getSiteMap() {
    return this.getXML();
  }
  pushPaths(paths) {
    for (const path of paths) {
      this.urls.push(`${path}`);
    }
  }
  getXML() {
    this.urls = [...new Set(this.urls)].filter((v) => this.excludeAnyRegExp?.test(v) !== true);
    const items = `	<url><loc>${this.basePath}` + this.urls.join(`</loc></url>
	<url><loc>${this.basePath}`) + "</loc></url>";
    let xml = '<?xml version="1.0" encoding="UTF-8"?>\n';
    xml += '<urlset xmlns="https://www.sitemaps.org/schemas/sitemap/0.9">\n';
    xml += `${items}
`;
    xml += "</urlset>";
    return xml;
  }
  generateUrls(route, data) {
    const urls = [];
    if (data) {
      for (const o of data) {
        if (Array.isArray(o) && o.length > 1 && Array.isArray(o[1])) {
          urls.push(...this.generateUrls(route.replace("@", o[0]), o[1]));
        } else if (typeof o === "string") {
          urls.push(route.replace("@", o));
        }
      }
    }
    return urls;
  }
}

const lsDir = function(dir, files = []) {
  if (statSync(dir).isDirectory()) {
    readdirSync(dir).forEach((file) => {
      const absolute = join(dir, file);
      if (statSync(absolute).isDirectory()) {
        return lsDir(absolute, files);
      } else {
        files.push(absolute);
      }
    });
  }
  return files;
};
const module = defineNuxtModule({
  meta: {
    name: "nuxt-mapple",
    version: "0.2.0",
    configKey: "mapple",
    compatibility: { nuxt: "^3.0.0" }
  },
  defaults: {
    basePath: "",
    dynamicRoutes: [],
    staticRoutes: [],
    verbose: false,
    useContent: false,
    excludeContent: void 0,
    excludeAny: void 0
  },
  setup: function(options, nuxt) {
    let sitemapRoutes;
    let pages = [];
    let contentPaths = [];
    const resolver = createResolver(import.meta.url);
    const filePath = resolver.resolve(
      nuxt.options.srcDir,
      "node_modules/.cache/.sitemap/sitemap.xml"
    );
    let excludeAnyRegExp;
    if (options.useContent) {
      const dir = resolver.resolve(
        nuxt.options.srcDir,
        "content"
      );
      const contentPath = join(nuxt.options.srcDir, "content");
      const files = lsDir(dir) || [];
      contentPaths = files.filter((el) => el.match(/\/[^.]*(\.md$|[[\]:])/)).map((el) => {
        return el.replace(contentPath, "").replace(/\.md$/, "");
      });
      if (options.excludeContent) {
        try {
          const excludeContentRegExp = new RegExp(options.excludeContent);
          contentPaths = contentPaths.filter((el) => excludeContentRegExp.test(el) !== true);
        } catch (err) {
          console.error(err);
        }
      }
      if (options.excludeAny) {
        try {
          excludeAnyRegExp = new RegExp(options.excludeAny);
          if (excludeAnyRegExp) {
            contentPaths = contentPaths.filter((el) => excludeAnyRegExp?.test(el) !== true);
          }
        } catch (err) {
          console.error(err);
        }
      }
    }
    nuxt.options.nitro.publicAssets = nuxt.options.nitro.publicAssets || [];
    nuxt.options.nitro.publicAssets.push({ baseURL: "/", dir: dirname(filePath) });
    nuxt.hook("pages:extend", (pagesExtend) => {
      pages = [...pages, ...pagesExtend];
      const sitemapRoutesOrig = pages.map((route) => route.path);
      sitemapRoutes = sitemapRoutesOrig.filter((r) => !r.includes(":"));
      sitemapRoutes = [...sitemapRoutes, ...options?.staticRoutes || []].filter((v) => !!v);
      const generator = new SiteMapGenerator(options?.dynamicRoutes, { ...options, excludeAnyRegExp });
      if (options.useContent) {
        generator.pushPaths(contentPaths);
      }
      generator.pushPaths(sitemapRoutes);
      const sitemap = generator.getSiteMap();
      mkdirSync(dirname(filePath), { recursive: true });
      writeFileSync(filePath, sitemap);
      console.log(`\u{1F310}\uFE0F Sitemap created (${generator.urlCount} URLs${generator.urlCount === 0 ? "... did you forget your config?" : ""})`);
      if (options.verbose) {
        console.log("-----------------------");
        console.log(sitemap);
        console.log("-----------------------");
      }
    });
  }
});

export { module as default };
